package edu.ucsd.act.demo.web;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.client.support.HttpAccessor;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

import edu.ucsd.act.demo.dao.PostDAO;
import edu.ucsd.act.demo.domain.Post;
import edu.ucsd.act.demo.service.PostService;
import edu.ucsd.act.demo.web.validator.PostValidator;

/**
 * @author haforoozanfard
 * Spring MVC controller that handles CRUD requests for Post entities
 *
 */
@Controller("PostController")
@SessionAttributes({"posts"})
public class PostController {

	/**
	 * DAO injected by Spring that manages Post entities
	 *
	 */
	@Autowired
	private PostDAO postDAO;

	/**
	 * Service injected by Spring that provides CRUD operations for Post entities
	 *
	 */
	@Autowired
	private PostService postService;


	/**
	 * Show all Post entities
	 *
	 */
	@RequestMapping("/indexPost")
	public ModelAndView listPosts() {
		ModelAndView mav = new ModelAndView();
		mav.addObject("posts", postService.loadPosts());
		mav.setViewName("post/listPosts.jsp");
		return mav;
	}

	/**
	 * Select an existing Post entity
	 * @throws Exception
	 *
	 */
	@RequestMapping("/selectPost")
	public ModelAndView selectPost(
			@RequestParam Integer idKey,
			@ModelAttribute("posts") LinkedHashSet<Post> posts
			) throws Exception {
		ModelAndView mav = new ModelAndView();
		mav.addObject("post", postDAO.findPostById(idKey, posts));
		mav.setViewName("post/viewPost.jsp");
		return mav;
	}

	/**
	 * JSON Validate :)
	 * @param post
	 * @param bindingResult
	 * @return
	 */
	@RequestMapping("/postJsonValidator")
	public 	ModelAndView postJsonValidator(
			@ModelAttribute("post") Post post
			,BindingResult bindingResult
	) {
		Map<String, String> error_collection = new HashMap<String, String>();
		ModelAndView mav = new ModelAndView();
		// validate
		new PostValidator().validate(post, bindingResult);
		if (bindingResult.hasErrors()) {
			List<ObjectError> errs = bindingResult.getAllErrors();
			for (ObjectError err : errs) {
				if(err instanceof FieldError){
					FieldError fielderr = (FieldError)err;
					error_collection.put(fielderr.getField(),err.getDefaultMessage());
				}else{
					error_collection.put("objecterr."+err.getObjectName(),err.getDefaultMessage());
				}
			}
			mav.addObject("errors", error_collection);
		}
		return mav;
	}

	/**
	 * Save an existing Post entity
	 *
	 */
	@RequestMapping("/savePost")
	public ModelAndView savePost(
			HttpSession session,
			@ModelAttribute("post") Post post
			,BindingResult bindingResult
			) {
		ModelAndView mav = new ModelAndView();
		// validate
		new PostValidator().validate(post, bindingResult);
		LinkedHashSet<Post> posts = (LinkedHashSet<Post>)session.getAttribute("posts");
		if (bindingResult.hasErrors()) {
			mav.setViewName("post/viewPost.jsp");
		}else{
			Set<Post> updated_posts = postService.savePost(post, posts);
			mav.setViewName("post/listPosts.jsp");
			mav.addObject("posts", updated_posts);
			mav.addObject("success", "Post Saved Successfully!");
		}
		return mav;
	}

}